@@ -4,10 +4,8 @@
  *
  *    Graphics Subsystem interface
  *
- *  Copyright 1999 - The FreeType Development Team - www.freetype.org
- *
- *
- *
+ *  Copyright 1999, 2000, 2001, 2002
+ *     - The FreeType Development Team - www.freetype.org
  *
  ***************************************************************************/
 
@@ -50,6 +48,8 @@
     gr_pixel_mode_rgb565,      /* 16-bits mode - 65536 colors      */
     gr_pixel_mode_rgb24,       /* 24-bits mode - 16 million colors */
     gr_pixel_mode_rgb32,       /* 32-bits mode - 16 million colors */
+    gr_pixel_mode_lcd,         /* horizontal RGB-decimated         */
+    gr_pixel_mode_lcdv,        /* vertical RGB-decimated           */
 
     gr_pixel_mode_max          /* don't remove */
 
@@ -163,6 +163,7 @@
   *    writes a given glyph bitmap to a target surface.
   *
   * <Input>
+  *    is_bgr  :: 1 if BGR, 0 if RGB
   *    target  :: handle to target bitmap
   *    glyph   :: handle to source glyph bitmap
   *    x       :: position of left-most pixel of glyph image in target surface
@@ -186,11 +187,13 @@
   *
   **********************************************************************/
 
-  extern int   grBlitGlyphToBitmap( grBitmap*  target,
-                                    grBitmap*  glyph,
-                                    grPos      x,
-                                    grPos      y,
-                                    grColor    color );
+  extern int
+  grBlitGlyphToBitmap( int        is_bgr,
+                       grBitmap*  target,
+                       grBitmap*  glyph,
+                       grPos      x,
+                       grPos      y,
+                       grColor    color );
 
 
  /**********************************************************************
