@@ -2,10 +2,10 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-1999 by                                                  */
+/*  Copyright 1996-1999, 2000, 2001, 2002 by                                */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
-/*  blitter.c: Support for blitting of bitmaps with various depth.          */
+/*  grblit.c: Support for blitting of bitmaps with various depth.           */
 /*                                                                          */
 /****************************************************************************/
 
@@ -33,6 +33,14 @@
       width = (width + 1) & -2;
       break;
 
+    case gr_pixel_mode_lcd:
+      width /= 3;
+      break;
+
+    case gr_pixel_mode_lcdv:
+      height /= 3;
+      break;
+
     default:
       ;
     }
@@ -182,7 +190,7 @@
       {
         unsigned char*  _read  = read;
         unsigned char*  _write = write;
-        unsigned char   old;
+        unsigned int    old;
         int             shift2 = (8-shift);
 
         if ( left_clip )
@@ -196,13 +204,13 @@
           unsigned char val;
 
           val = *_read++;
-          *_write++ |= ( (val >> shift) | old );
+          *_write++ |= (unsigned char)( (val >> shift) | old );
           old = val << shift2;
           x--;
         }
 
         if ( !blit->right_clip )
-          *_write |= old;
+          *_write |= (unsigned char)old;
 
         read  += blit->read_line;
         write += blit->write_line;
@@ -272,7 +280,7 @@
     int             x, y, phase,shift;
     unsigned char*  read;
     unsigned char*  write;
-    unsigned char   col;
+    unsigned int    col;
 
 
     col   = color.value & 15;
@@ -296,13 +304,13 @@
       {
         if (val & 0x10000)
           val = *_read++ | 0x100;
-           
+
         if ( val & 0x80 )
         {
           if ( _phase )
-            *_write = (*_write & 0xF0) | col;
+            *_write = (unsigned char)((*_write & 0xF0) | col);
           else
-            *_write = (*_write & 0x0F) | (col << 4);
+            *_write = (unsigned char)((*_write & 0x0F) | (col << 4));
         }
 
         val <<= 1;
@@ -375,7 +383,7 @@
   void  blit_mono_to_rgb24( grBlitter*  blit,
                             grColor     color )
   {
-    int             x, y,shift;
+    int             x, y, shift;
     unsigned char*  read;
     unsigned char*  write;
 
@@ -658,8 +666,8 @@
       conv->table        = table;
 
       for ( n = 0; n < source_grays; n++ )
-        ((unsigned char*)table)[n] = (unsigned char)(n*(target_grays-1)) /
-                                         (source_grays-1);
+        ((unsigned char*)table)[n] = (unsigned char)(n*(target_grays-1) /
+                                         (source_grays-1));
 
       gr_num_conversions++;
       gr_last_conversion = conv;
@@ -786,40 +794,44 @@
 
 
 
-#define  compose_pixel( a, b, n, max )       \
-   {                                         \
-     int  d, half = max >> 1;                \
-                                             \
-     d = (int)b.chroma[0] - a.chroma[0];     \
-     a.chroma[0] += (n*d + half)/max;        \
-                                             \
-     d = (int)b.chroma[1] - a.chroma[1];     \
-     a.chroma[1] += (n*d + half)/max;        \
-                                             \
-     d = (int)b.chroma[2] - a.chroma[2];     \
-     a.chroma[2] += (n*d + half)/max;        \
-   }
+#define compose_pixel_full( a, b, n0, n1, n2, max )          \
+  {                                                          \
+    int  d, half = max >> 1;                                 \
+                                                             \
+                                                             \
+    d = (int)b.chroma[0] - a.chroma[0];                      \
+    a.chroma[0] += (unsigned char)((n0*d + half)/max);       \
+                                                             \
+    d = (int)b.chroma[1] - a.chroma[1];                      \
+    a.chroma[1] += (unsigned char)((n1*d + half)/max);       \
+                                                             \
+    d = (int)b.chroma[2] - a.chroma[2];                      \
+    a.chroma[2] += (unsigned char)((n2*d + half)/max);       \
+  }
+
+#define compose_pixel( a, b, n, max )  \
+    compose_pixel_full( a, b, n, n, n, max )
 
 
-#define  extract555( pixel, color )           \
-   color.chroma[0] = (pixel >> 10) & 0x1F;  \
-   color.chroma[1] = (pixel >>  5) & 0x1F;  \
-   color.chroma[2] = (pixel      ) & 0x1F;
+#define extract555( pixel, color )                           \
+   color.chroma[0] = (unsigned char)((pixel >> 10) & 0x1F);  \
+   color.chroma[1] = (unsigned char)((pixel >>  5) & 0x1F);  \
+   color.chroma[2] = (unsigned char)((pixel      ) & 0x1F);
 
 
-#define  extract565( pixel, color )           \
-   color.chroma[0] = (pixel >> 11) & 0x1F;  \
-   color.chroma[1] = (pixel >>  5) & 0x3F;  \
-   color.chroma[2] = (pixel      ) & 0x1F;
+#define extract565( pixel, color )                           \
+   color.chroma[0] = (unsigned char)((pixel >> 11) & 0x1F);  \
+   color.chroma[1] = (unsigned char)((pixel >>  5) & 0x3F);  \
+   color.chroma[2] = (unsigned char)((pixel      ) & 0x1F);
 
 
-#define  inject555( color )                           \
+#define inject555( color )                          \
    ( ( (unsigned short)color.chroma[0] << 10 ) |    \
      ( (unsigned short)color.chroma[1] <<  5 ) |    \
        color.chroma[2]                         )
 
 
-#define  inject565( color )         \
+#define inject565( color )                          \
    ( ( (unsigned short)color.chroma[0] << 11 ) |    \
      ( (unsigned short)color.chroma[1] <<  5 ) |    \
        color.chroma[2]                         )
@@ -877,7 +889,7 @@
             extract555( pix16, pix );
 
             compose_pixel( pix, color, val, max );
-            *pixel = inject555(pix);
+            *pixel = (unsigned short)(inject555(pix));
           }
         }
         _write += 2;
@@ -944,7 +956,7 @@
             extract565( pix16, pix );
 
             compose_pixel( pix, color, val, max );
-            *pixel = inject565( pix );
+            *pixel = (short)inject565( pix );
           }
         }
         _write +=2;
@@ -1095,6 +1107,172 @@
   }
 
 
+/**************************************************************************/
+/*                                                                        */
+/* <Function> blit_lcd_to_24                                              */
+/*                                                                        */
+/**************************************************************************/
+
+  static void
+  blit_lcd_to_24( int         is_bgr,
+                  grBlitter*  blit,
+                  grColor     color,
+                  int         max )
+  {
+    int             y;
+    unsigned char*  read;
+    unsigned char*  write;
+
+    read   = blit->read  + 3*blit->xread;
+    write  = blit->write + 3*blit->xwrite;
+
+    y = blit->height;
+    do
+    {
+      unsigned char*  _read  = read;
+      unsigned char*  _write = write;
+      int             x      = blit->width;
+
+      while (x > 0)
+      {
+        int    val0, val1, val2;
+
+        if (is_bgr)
+        {
+          val0 = _read[0];
+          val1 = _read[1];
+          val2 = _read[2];
+        }
+        else
+        {
+          val0 = _read[2];
+          val1 = _read[1];
+          val2 = _read[0];
+        }
+
+        if ( val0 | val1 | val2 )
+        {
+          if ( val0 == val1 &&
+               val0 == val2 &&
+               val0 == max  )
+          {
+            _write[0] = color.chroma[0];
+            _write[1] = color.chroma[1];
+            _write[2] = color.chroma[2];
+          }
+          else
+          {
+            /* compose gray value */
+            grColor pix;
+
+            pix.chroma[0] = _write[0];
+            pix.chroma[1] = _write[1];
+            pix.chroma[2] = _write[2];
+
+            compose_pixel_full( pix, color, val0, val1, val2, max );
+
+            _write[0] = pix.chroma[0];
+            _write[1] = pix.chroma[1];
+            _write[2] = pix.chroma[2];
+          }
+        }
+        _write += 3;
+        _read  += 3;
+        x--;
+      }
+
+      read  += blit->read_line;
+      write += blit->write_line;
+      y--;
+    }
+    while (y > 0);
+  }
+
+
+/**************************************************************************/
+/*                                                                        */
+/* <Function> blit_lcdv_to_24                                             */
+/*                                                                        */
+/**************************************************************************/
+
+  static void
+  blit_lcdv_to_24( int         is_bgr,
+                   grBlitter*  blit,
+                   grColor     color,
+                   int         max )
+  {
+    int             y;
+    unsigned char*  read;
+    unsigned char*  write;
+    long            line;
+
+    read   = blit->read  + blit->xread;
+    write  = blit->write + 3*blit->xwrite;
+    line   = blit->read_line;
+
+    y = blit->height;
+    do
+    {
+      unsigned char*  _read  = read;
+      unsigned char*  _write = write;
+      int             x      = blit->width;
+
+      while (x > 0)
+      {
+        unsigned char    val0, val1, val2;
+
+        if (is_bgr)
+        {
+          val0 = _read[2*line];
+          val1 = _read[1*line];
+          val2 = _read[0*line];
+        }
+        else
+        {
+          val0 = _read[0*line];
+          val1 = _read[1*line];
+          val2 = _read[2*line];
+        }
+
+        if ( val0 | val1 | val2 )
+        {
+          if ( val0 == val1 &&
+               val0 == val2 &&
+               val0 == max  )
+          {
+            _write[0] = color.chroma[0];
+            _write[1] = color.chroma[1];
+            _write[2] = color.chroma[2];
+          }
+          else
+          {
+            /* compose gray value */
+            grColor pix;
+
+            pix.chroma[0] = _write[0];
+            pix.chroma[1] = _write[1];
+            pix.chroma[2] = _write[2];
+
+            compose_pixel_full( pix, color, val0, val1, val2, max );
+
+            _write[0] = pix.chroma[0];
+            _write[1] = pix.chroma[1];
+            _write[2] = pix.chroma[2];
+          }
+        }
+        _write += 3;
+        _read  += 1;
+        x--;
+      }
+
+      read  += 3*line;
+      write += blit->write_line;
+      y--;
+    }
+    while (y > 0);
+  }
+
+
  /**********************************************************************
   *
   * <Function>
@@ -1125,6 +1303,7 @@
     0,
     0,
     0,
+    0,
     blit_gray_to_555,
     blit_gray_to_565,
     blit_gray_to_24,
@@ -1132,24 +1311,26 @@
   };
 
 
-  extern int  grBlitGlyphToBitmap( grBitmap*  target,
-                                   grBitmap*  glyph,
-                                   grPos      x,
-                                   grPos      y,
-                                   grColor    color )
+  extern int
+  grBlitGlyphToBitmap( int        is_bgr,
+                       grBitmap*  target,
+                       grBitmap*  glyph,
+                       grPos      x,
+                       grPos      y,
+                       grColor    color )
   {
     grBlitter    blit;
     grPixelMode  mode;
 
+
     /* check arguments */
-    if (!target || !glyph)
+    if ( !target || !glyph )
     {
       grError = gr_err_bad_argument;
       return -1;
     }
 
-
-    /* set up blitter and compute clipping. Return immediately if needed */
+    /* set up blitter and compute clipping.  Return immediately if needed */
     blit.source = *glyph;
     blit.target = *target;
     mode        = target->mode;
@@ -1157,10 +1338,9 @@
     if ( compute_clips( &blit, x, y ) )
       return 0;
 
-
-    /* handle monochrome bitmap blitting */
-    if (glyph->mode == gr_pixel_mode_mono)
+    switch ( glyph->mode )
     {
+    case gr_pixel_mode_mono:     /* handle monochrome bitmap blitting */
       if ( mode <= gr_pixel_mode_none || mode >= gr_pixel_mode_max )
       {
         grError = gr_err_bad_source_depth;
@@ -1168,70 +1348,86 @@
       }
 
       gr_mono_blitters[mode]( &blit, color );
-      goto End;
-    }
+      break;
 
-    /* handle gray bitmap composition */
-    if (glyph->mode == gr_pixel_mode_gray &&
-        glyph->grays > 1                  )
-    {
-      int          target_grays = target->grays;
-      int          source_grays = glyph->grays;
-      const byte*  saturation;
-
-      if ( mode == gr_pixel_mode_gray && target_grays > 1 )
+    case gr_pixel_mode_gray:
+      if ( glyph->grays > 1 )
       {
-        /* rendering into a gray target - use special composition */
-        /* routines..                                             */
-        if ( gr_last_saturation->count == target_grays )
-          saturation = gr_last_saturation->table;
-        else
+        int          target_grays = target->grays;
+        int          source_grays = glyph->grays;
+        const byte*  saturation;
+
+
+        if ( mode == gr_pixel_mode_gray && target_grays > 1 )
         {
-          saturation = grGetSaturation( target_grays );
-          if (!saturation) return -3;
-        }
-
-
-        if ( target_grays == source_grays )
-          blit_gray_to_gray_simple( &blit, saturation );
-        else
-        {
-          const byte*  conversion;
-
-          if ( gr_last_conversion->target_grays == target_grays &&
-               gr_last_conversion->source_grays == source_grays )
-            conversion = gr_last_conversion->table;
+          /* rendering into a gray target - use special composition */
+          /* routines..                                             */
+          if ( gr_last_saturation->count == target_grays )
+            saturation = gr_last_saturation->table;
           else
           {
-            conversion = grGetConversion( target_grays, source_grays );
-            if (!conversion) return -3;
-          };
+            saturation = grGetSaturation( target_grays );
+            if ( !saturation )
+              return -3;
+          }
 
-          blit_gray_to_gray( &blit, saturation, conversion );
+          if ( target_grays == source_grays )
+            blit_gray_to_gray_simple( &blit, saturation );
+          else
+          {
+            const byte*  conversion;
+
+
+            if ( gr_last_conversion->target_grays == target_grays &&
+                 gr_last_conversion->source_grays == source_grays )
+              conversion = gr_last_conversion->table;
+            else
+            {
+              conversion = grGetConversion( target_grays, source_grays );
+              if ( !conversion )
+                return -3;
+            }
+
+            blit_gray_to_gray( &blit, saturation, conversion );
+          }
         }
-      }
-      else
-      {
-        /* rendering into a color target */
-        if ( mode <= gr_pixel_mode_gray ||
-             mode >= gr_pixel_mode_max  )
+        else
         {
-          grError = gr_err_bad_target_depth;
-          return -1;
-        }
+          /* rendering into a color target */
+          if ( mode <= gr_pixel_mode_gray ||
+               mode >= gr_pixel_mode_max  )
+          {
+            grError = gr_err_bad_target_depth;
+            return -1;
+          }
 
-        gr_color_blitters[mode]( &blit, color, source_grays-1 );
+          gr_color_blitters[mode]( &blit, color, source_grays - 1 );
+        }
       }
-      goto End;
+      break;
+
+    case gr_pixel_mode_lcd:
+      if ( glyph->grays > 1 && mode == gr_pixel_mode_rgb24 )
+      {
+        blit_lcd_to_24( is_bgr, &blit, color, glyph->grays-1 );
+        break;
+      }
+
+    case gr_pixel_mode_lcdv:
+      if ( glyph->grays > 1 && mode == gr_pixel_mode_rgb24 )
+      {
+        blit_lcdv_to_24( is_bgr, &blit, color, glyph->grays-1 );
+        break;
+      }
+
+    default:
+      /* we don't support the blitting of bitmaps of the following  */
+      /* types : pal4, pal8, rgb555, rgb565, rgb24, rgb32           */
+      /*                                                            */
+      grError = gr_err_bad_source_depth;
+      return -2;
     }
 
-    /* we don't support the blitting of bitmaps of the following  */
-    /* types : pal4, pal8, rgb555, rgb565, rgb24, rgb32           */
-    /*                                                            */
-    grError = gr_err_bad_source_depth;
-    return -2;
-
-  End:
     return 0;
   }
 
